// ++
//      Affine Transform                                                     AffineTransform.mq4 
//           ( ,       
//     Applied_Price),     .                
//    :                                                                      
//      ,    ;                                        
//       ,       Trendline_Name;  
//         Applied_Price (   =7     
//         ).                                                         
//   :     ,    .       
//      ,    :  ..       
//       MQL4 Code Base: http://codebase.mql4.com/ru/6280                      
//          (.mq4),        
//   .                                                                             
// ----------------------------------------------------------------------------------------------
//      [release 2] 10.12.2009                                                                   
//  (+)  ,     MaxBars. . :        
//      0: ;                                                                            
//     -1:      .                                          
//  (+)     .  ,   Applied_Price = 7. 
//       ,    :    ,              
//     .       /.    
//  (+)       ,    .  
//      [release 1] 06.12.2009                                                                   
//  () .    .  _________________________________
//                                                             |  xp3rienced, Ekaterinburg 2009 
// ++
#property copyright " xp3rienced"
#property link      "no4ta[at]inbox.ru"

#define PRICE_HIGHLOW 7

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Green
#property indicator_color2 Green
#property indicator_width1 1
#property indicator_width2 1

//====[   ]====
extern string Trendline_Name = "A_Line";    //   ( )
extern int Applied_Price = PRICE_CLOSE;     //  ,     . :
// Close = 0; Open = 1; High = 2; Low = 3; Median = 4; Typical = 5; Weighted = 6; HighLow = 7
extern int MaxBars = 500;                   //   . 0 = ; -1 =     

//====[   ]====
double At_High[];
double At_Low[];

//====[    ]====
double LineAngleTg;                         //     
double LinePrice1, LinePrice2;              //     
datetime LineTime1, LineTime2;              //     
int Shift1, Shift2;                         //     
int Multiplier;                             // ,    (  )
bool NeedUpdate;

//+------------------------------------------------------------------+
//|                                    |
//+------------------------------------------------------------------+
int init()
{
//----
  if((Applied_Price < 0) || (Applied_Price > 7)) Applied_Price = PRICE_CLOSE;
//----   
  SetIndexBuffer(0, At_High);
  SetIndexBuffer(1, At_Low);
//----  
  color c = GetTrendLineColor();
  SetIndexStyle(0, DRAW_LINE, EMPTY, EMPTY, c);
  SetIndexStyle(1, DRAW_LINE, EMPTY, EMPTY, c);
//----    ,    
  IndicatorShortName(StringConcatenate("Affine Transform(", PriceConstToStr(Applied_Price), ")"));
  string label1 = PriceConstToStr(Applied_Price);
  string label2 = label1;
  if(Applied_Price == 7)
  {
    label1 = "High";
    label2 = "Low";
  }
  SetIndexLabel(0, label1);
  SetIndexLabel(1, label2);
//----
  NeedUpdate = true;
  Multiplier = MathPow(10, Digits);
//----  
  return(0);
}

//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
int deinit()
{
//----
  
//----
  return(0);
}

//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
int start()
{
  int limit = Bars - IndicatorCounted();
  if(limit < Bars) limit++;
//----    ,     -  
  CalcAngleTg();
  if(NeedUpdate)
  {
    limit = Bars - 1;
    NeedUpdate = false;
  }
//----   ,  
  if((MaxBars > 0) && (MaxBars < limit)) limit = MaxBars;
  if(MaxBars == -1) limit = Shift1;
//----   
  if(Applied_Price == 7)
  {
//----       
    for(int i=0; i<limit; i++)
    {
      At_High[i] = CoordTransform(PRICE_HIGH, i);
      At_Low[i] = CoordTransform(PRICE_LOW, i);
    }
    return(0);
  }
//----    ,    
  for(i=0; i<limit; i++)
  {
    At_High[i] = CoordTransform(Applied_Price, i);
    At_Low[i] = At_High[i];
  }
//----
  return(0);
}
//+------------------------------------------------------------------+

void CalcAngleTg()
{
  if(ObjectFind(Trendline_Name) == -1) return;
//----
  double tg;
//----
  LinePrice1 = ObjectGet(Trendline_Name, OBJPROP_PRICE1);
  LinePrice2 = ObjectGet(Trendline_Name, OBJPROP_PRICE2);
  LineTime1 = ObjectGet(Trendline_Name, OBJPROP_TIME1);
  LineTime2 = ObjectGet(Trendline_Name, OBJPROP_TIME2);
//----
  Shift1 = iBarShift(NULL, 0, LineTime1, true);
  Shift2 = iBarShift(NULL, 0, LineTime2, true);
//----
  tg = (LinePrice2 - LinePrice1) * Multiplier / (GetShift(Shift2) - GetShift(Shift1));
  if(tg != LineAngleTg) NeedUpdate = true;
  LineAngleTg = tg;
//----
  //Print("tg(angle) = ", LineAngleTg);
}

double CoordTransform(int price, int shift)
{
  return((GetPrice(price, shift) - LinePrice1) * Multiplier - (GetShift(shift) - GetShift(Shift1)) * LineAngleTg);
}

double GetPrice(int price_const, int shift)
{
  switch(price_const)
  {
    case PRICE_CLOSE: return(Close[shift]);
    case PRICE_OPEN: return(Open[shift]);
    case PRICE_HIGH: return(High[shift]);
    case PRICE_LOW: return(Low[shift]);
    case PRICE_MEDIAN: return((High[shift] + Low[shift]) / 2);
    case PRICE_TYPICAL: return((High[shift] + Low[shift] + Close[shift]) / 3);
    case PRICE_WEIGHTED: return((High[shift] + Low[shift] + Close[shift]*2) / 4);
  }
}

int GetShift(int index)
{
  return(Bars-1 - index);
}

color GetTrendLineColor()
{
  return(ObjectGet(Trendline_Name, OBJPROP_COLOR));
}

string PriceConstToStr(int price_const)
{
  switch(price_const)
  {
    case PRICE_CLOSE: return("Close");
    case PRICE_OPEN: return("Open");
    case PRICE_HIGH: return("High");
    case PRICE_LOW: return("Low");
    case PRICE_MEDIAN: return("Median price");
    case PRICE_TYPICAL: return("Typical price");
    case PRICE_WEIGHTED: return("Weighted price");
    case PRICE_HIGHLOW: return("HighLow");
  }
}
//+------------------------------------------------------------------+